const localhost = 'nanobrow-layout.loc';

const srcDir    = './src';
const distDir   = './dist';
const publicDir = './public';

const scssSrcDir = srcDir + '/scss';
const cssSrcDir  = srcDir + '/css';
const jsSrcDir   = srcDir + '/js';

const cssDistDir    = distDir + '/assets/css';
const cssMinDistDir = distDir + '/assets/css-min';
const jsDistDir     = distDir + '/assets/js';

const webpackModeDev   = false;
const useBrowserSync   = true;
const minifyCssWithAll = false;

const gulp         = require('gulp');
const sass         = require('gulp-sass')(require('sass'));
const sourcemaps   = require('gulp-sourcemaps');
const rename       = require('gulp-rename');
const autoprefixer = require('gulp-autoprefixer');
const webpack      = require('webpack-stream');
const del          = require('del');
const strip        = require('gulp-strip-comments');
const named        = require('vinyl-named');
const browserSync  = require('browser-sync').create();
const plumber      = require('gulp-plumber');
const cleanCSS     = require('gulp-clean-css');

function copyTemplates() {

	del([`${distDir}/**`, `!${distDir}/assets`], { force: true });

	return gulp.src([`${publicDir}/*.*`, `!${publicDir}/assets`])
		.pipe(plumber())
		.pipe(gulp.dest(distDir));
	
}

function watchTemplates() {

	return gulp.watch([`${publicDir}/**`, `!${publicDir}/assets`], gulp.series('copyTemplates', browserSyncReload));
	
}

function copyAssets() {

	del([`${distDir}/assets/**`, `!${distDir}/assets/css`, `!${distDir}/assets/css-min`, `!${distDir}/assets/js`], { force: true });

	return gulp.src(`${publicDir}/assets/**`)
		.pipe(plumber())
		.pipe(gulp.dest(`${distDir}/assets`));
		
}

function watchAssets() {

	return gulp.watch(`${publicDir}/assets/**`, gulp.series('copyAssets', browserSyncReload));

}

function scssCompile() {

	del(`${cssDistDir}/**`, { force: true });

	gulp.src(`${scssSrcDir}/*.scss`)
		.pipe(gulp.dest(`${cssDistDir}/scss`));

	gulp.src(`${scssSrcDir}/*.scss`)
		.pipe(plumber())
		.pipe(sourcemaps.init())
		.pipe(sass({
			outputStyle: 'compressed',
		}))
		.pipe(autoprefixer({
			cascade: false
		}))
		.pipe(rename(function(path){
			path.basename += '.min';
		}))
		.pipe(sourcemaps.write('.'))
		.pipe(gulp.dest(cssDistDir));

	return gulp.src(`${scssSrcDir}/*.scss`)
		.pipe(sourcemaps.init())
		.pipe(sass({
			outputStyle: 'expanded',
		}))
		.pipe(autoprefixer({
			cascade: false
		}))
		.pipe(sourcemaps.write('.'))
		.pipe(gulp.dest(cssDistDir));
	
}

function watchScss() {

	return gulp.watch(`${scssSrcDir}/**/*.scss`, gulp.series('scssCompile', browserSyncReload));

}

function cssMinify() {

	del(`${cssMinDistDir}/**`, { force: true });
	
	gulp.src(`${cssSrcDir}/**/*.css`)
		.pipe(sourcemaps.init())
		.pipe(cleanCSS())
		.pipe(autoprefixer({
			cascade: false
		}))
		.pipe(rename(function(path){
			path.basename += '.min';
		}))
		.pipe(sourcemaps.write('.'))
		.pipe(gulp.dest(cssMinDistDir));

	return gulp.src(`${cssSrcDir}/**/*.css`)
		.pipe(autoprefixer({
			cascade: false
		}))
		.pipe(gulp.dest(cssMinDistDir));
	
}

function watchCss() {

	return gulp.watch(`${cssSrcDir}/**/*.css`, gulp.series('cssMinify', browserSyncReload));

}

function webpackBuild(minimize, generateMap) {

	return {
		mode: webpackModeDev ? 'development' : 'production',
		output: {
			filename: minimize ? '[name].min.js' : '[name].js',
		},
		watch: false,
		devtool: generateMap ? 'source-map' : false,
		optimization: {
			minimize: minimize
		},
		module: {
			rules: [
				{
					test: /\.m?js$/,
					exclude: /(node_modules|bower_components)/,
					use: {
						loader: 'babel-loader',
						options: {
							presets: [
								[
									'@babel/preset-env',
									{
										debug: true,
										corejs: 3,
										useBuiltIns: 'usage',
									},
								],
							],
						},
					},
				},
			],
		},
	}
	
}

function jsBuild() {

	del(`${jsDistDir}/**`, { force: true });
	
	// JS min | without map
	gulp.src(`${jsSrcDir}/*.js`)
		.pipe(plumber())
		.pipe(named())
		.pipe(webpack(webpackBuild(true, false)))
		.pipe(strip({
			ignore: /\/\/(# sourceMappingURL).*/g
		}))
		.pipe(gulp.dest(jsDistDir));
	
	// JS not min | without map
	return gulp.src(`${jsSrcDir}/*.js`)
		.pipe(plumber())
		.pipe(named())
		.pipe(webpack(webpackBuild(false, false)))
		.pipe(strip({
			ignore: /\/\/(# sourceMappingURL).*/g
		}))
		.pipe(gulp.dest(jsDistDir));
	
}

function jsBuildDev() {

	del(`${jsDistDir}/**`, { force: true });

	// JS min | with map
	gulp.src(`${jsSrcDir}/*.js`)
		.pipe(plumber())
		.pipe(named())
		.pipe(webpack(webpackBuild(true, true)).on('error', (error) => {return;}))
		.pipe(strip({
			ignore: /\/\/(# sourceMappingURL).*/g
		}))
		.pipe(gulp.dest(jsDistDir));
	
	// JS not min | with map
	return gulp.src(`${jsSrcDir}/*.js`)
		.pipe(plumber())
		.pipe(named())
		.pipe(webpack(webpackBuild(false, true)).on('error', (error) => {return;}))
		.pipe(strip({
			ignore: /\/\/(# sourceMappingURL).*/g
		}))
		.pipe(gulp.dest(jsDistDir));
	
}

function watchJs() {

	return gulp.watch(`${jsSrcDir}/*.js`, gulp.series('jsBuild', browserSyncReload));

}

function watchJsDev() {

	return gulp.watch(`${jsSrcDir}/*.js`, gulp.series('jsBuildDev', browserSyncReload));
	
}

function clearAll() {

	return del([`${distDir}/**`], { force: true });

}

function browserSyncInit() {

	if (useBrowserSync) {

		return browserSync.init({
			open: false,
			host: localhost,
			proxy: localhost,
			port: 8081
			// server: {
			// 	baseDir: "../"
			// }
		});
		
	} else return;

};

function browserSyncReload(cb) {

	if (useBrowserSync) browserSync.reload();
	cb();

}

gulp.task('copyTemplates', copyTemplates);
gulp.task('copyAssets',    copyAssets);
gulp.task('scssCompile',   scssCompile);
gulp.task('cssMinify',     cssMinify);
gulp.task('jsBuild',       jsBuild);
gulp.task('jsBuildDev',    jsBuildDev);

gulp.task('watchTemplates', watchTemplates);
gulp.task('watchAssets',    watchAssets);
gulp.task('watchScss',      watchScss);
gulp.task('watchCss',       watchCss);
gulp.task('watchJs',        watchJs);
gulp.task('watchJsDev',     watchJsDev);

gulp.task('clearAll', clearAll);

gulp.task('compileAll',    gulp.series(copyTemplates, copyAssets, scssCompile, cssMinify, jsBuild));
gulp.task('compileAllDev', gulp.series(copyTemplates, copyAssets, scssCompile, cssMinify, jsBuildDev));

gulp.task('watch', function() {

	browserSyncInit();
	watchTemplates();
	watchAssets();
	watchScss();
	if (minifyCssWithAll) watchCss();
	return watchJs();
	
});

gulp.task('watchDev', function() {

	browserSyncInit();
	watchTemplates();
	watchAssets();
	watchScss();
	if (minifyCssWithAll) watchCss();
	return watchJsDev();
	
});

gulp.task('start', gulp.series('clearAll', 'copyTemplates', 'copyAssets', 'scssCompile', 'jsBuild', function() {

	if (minifyCssWithAll) cssMinify();
	browserSyncInit();
	watchTemplates();
	watchAssets();
	watchScss();
	return watchJs();
	
}));

gulp.task('startDev', gulp.series('clearAll', 'copyTemplates', 'copyAssets', 'scssCompile', 'jsBuildDev', function() {
	
	if (minifyCssWithAll) cssMinify();
	browserSyncInit();
	watchTemplates();
	watchAssets();
	watchScss();
	return watchJsDev();
	
}));